"""Implements page video extended content model"""

from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from pages.conf import settings
from pages.models.pagebasecontent import PageBaseContent

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
    from embed_video.fields import EmbedVideoField

    class PageVideoContent(PageBaseContent):
        video = EmbedVideoField(blank=True)
        title = models.CharField(max_length=160, blank=True)
        description = models.TextField(max_length=160, blank=True)
        objects = models.Manager()

        def update_fields(self, change):
            if not change:
                self.type = 'video'
                self.is_extended = True

        class Meta(PageBaseContent.Meta):
            app_label = 'extpages'
            verbose_name = _('Video')
            verbose_name_plural = _('Video')

        class PageMeta(PageBaseContent.PageMeta):
            context_name = 'video'
            multiple_per_page = True
            multiple_per_locale = True



from pages.admin.page import PageAdmin
from pages.admin.pagecontent import PageContentAdmin
from pages.admin.pagecontenttype import PageContentTypeAdmin
from pages.admin.pagecontenttypes import PageSlugContentAdmin
from pages.admin.pagecontenttypes import PageRedirectContentAdmin
from pages.admin.pagecontenttypes import PageMetaContentAdmin
from pages.admin.pagecontenttypes import PageTextContentAdmin



# -*- coding: utf-8
from __future__ import unicode_literals
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.utils import translation

from pages.cache import cache
from pages.conf import settings
from pages.tests.base import PagesCase
from pages.models import PageSlugContent, PageMetaContent
from pages.models import PageRedirectContent
from pages.models import PageTextContent, PageMarkdownContent
from pages.views import PageDetailsView


class TestPages(PagesCase):

    def test_page_slug_model(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        obj = PageSlugContent.objects.filter(page=self.page_foo, language='en', slug='test')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:slug:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:slug:1')

    def test_page_meta_model(self):
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        obj = PageMetaContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:meta:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:meta:1')

    def test_page_redirect_model(self):
        PageRedirectContent.objects.create(page=self.page_foo)
        obj = PageRedirectContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:redirect:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:redirect:1')

    def test_page_text_model(self):
        PageTextContent.objects.create(page=self.page_foo, text='test')
        obj = PageTextContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:text:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:text:1')

    def test_page_markdown_model(self):
        PageMarkdownContent.objects.create(page=self.page_foo)
        obj = PageMarkdownContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:markdown:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:markdown:1')

    def test_page_absolute_url(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        self.page_foo.template = 'pages/page_text.html'
        self.page_foo.save()
        page_url = self.page_foo.get_absolute_url()
        self.assertEqual(page_url, '/en/page/test/')
        self.page_foo.delete()
        cache.clear()

    def test_page_text_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        self.page_foo.template = 'pages/page_text.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_text_view_cache(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        self.page_foo.template = 'pages/page_text.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        page_cache_key = settings.PAGES_PAGE_CACHE_KEY + 'en' + ':' + 'test' + ':' + 'False'
        page_cache_version_key = settings.PAGES_PAGE_VERSION_KEY + 'en' + ':' + 'test'
        cache_version = cache.get(page_cache_version_key)
        page = cache.get(page_cache_key, version=cache_version)
        self.assertNotEqual(page, None)
        self.page_foo.save()
        cache_version = cache.get(page_cache_version_key)
        self.assertEqual(cache_version, 2)
        self.page_foo.delete()
        cache.clear()

    def test_page_markdown_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageMarkdownContent.objects.create(page=self.page_foo, text='**test**')
        self.page_foo.template = 'pages/page_markdown.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_slug_uniques(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageSlugContent.objects.create(page=self.page_foo2, slug='test')
        PageSlugContent.objects.create(page=self.page_foo3, slug='test')
        PageSlugContent.objects.create(page=self.page_foo4, slug='test', language='de')
        obj = PageSlugContent.objects.filter(page=self.page_foo2, language='en', slug='test-1')[0]
        self.assertEqual(obj.slug, 'test-1')
        obj = PageSlugContent.objects.filter(page=self.page_foo3, language='en', slug='test-2')[0]
        self.assertEqual(obj.slug, 'test-2')
        obj = PageSlugContent.objects.filter(page=self.page_foo4, language='de', slug='test')[0]
        self.assertEqual(obj.slug, 'test')

    def test_page_slug_from_meta_title(self):
        PageMetaContent.objects.create(page=self.page_foo, title='test title')
        PageSlugContent.objects.create(page=self.page_foo, slug='')
        obj = PageSlugContent.objects.filter(page=self.page_foo, language='en', slug='test-title')[0]
        self.assertEqual(obj.slug, 'test-title')

    def test_page_slug_from_meta_intl_title(self):
        PageMetaContent.objects.create(page=self.page_foo5, title='prüfung title', language='de')
        PageSlugContent.objects.create(page=self.page_foo5, slug='', language='de')
        obj = PageSlugContent.objects.filter(page=self.page_foo5, language='de', slug='prüfung-title')[0]
        self.assertEqual(obj.slug, 'prüfung-title')

        PageMetaContent.objects.create(page=self.page_foo5, title='тест title', language='ru')
        PageSlugContent.objects.create(page=self.page_foo5, slug='', language='ru')
        obj = PageSlugContent.objects.filter(page=self.page_foo5, language='ru', slug='тест-title')[0]
        self.assertEqual(obj.slug, 'тест-title')

        settings.PAGES_PAGE_ONLY_ASCII_SLUGS = True
        PageMetaContent.objects.create(page=self.page_foo6, title='prüfung title', language='de')
        PageSlugContent.objects.create(page=self.page_foo6, slug='', language='de')
        obj = PageSlugContent.objects.filter(page=self.page_foo6, language='de', slug='prufung-title')[0]
        self.assertEqual(obj.slug, 'prufung-title')

        PageMetaContent.objects.create(page=self.page_foo6, title='тест title', language='ru')
        PageSlugContent.objects.create(page=self.page_foo6, slug='', language='ru')
        obj = PageSlugContent.objects.filter(page=self.page_foo6, language='ru', slug='test-title')[0]
        self.assertEqual(obj.slug, 'test-title')
        settings.PAGES_PAGE_ONLY_ASCII_SLUGS = False

    def test_page_with_non_ascii_slug(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='prüfung')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        self.page_foo.template = 'pages/page_text.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'prüfung'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='prüfung')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_fallback_language(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        self.page_foo.template = 'pages/page_text.html'
        self.page_foo.save()
        page_url = '/de' + reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('de')
        response = view(request=request, context=context, slug='test')
        self.assertEqual(response.status_code, 302)
        page_url = response.get('location')
        response = self.client.get(page_url)
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()



"""Implements admin interface for Pages"""

import importlib

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from mptt.admin import MPTTModelAdmin
from guardian.admin import GuardedModelAdmin

from pages.conf import settings
from pages.models import Page
from pages.models import PageContent
from pages.models import PageContentType
from pages.admin import pagecontenttypes
from pages.admin.pagecontent import PageContentInline


PAGE_EXT_CONTENT_INLINES = []

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
    if settings.PAGES_PAGE_EXT_CONTENT_INLINES is not None:
        try:
            for inline in settings.PAGES_PAGE_EXT_CONTENT_INLINES:
                PAGE_EXT_CONTENT_INLINES.append(importlib.import_module(inline))
        except ImportError as e:
            raise Exception('Extended content type inline import error: {0}'.format(e))


class PageAdmin(GuardedModelAdmin, MPTTModelAdmin):
    change_form_template = 'admin/pages/page/change_form.html'
    list_display = ['__str__',
                    'ptype', 'date_created_short', 'date_updated_short', 'date_approved_short',
                    'date_publication_short', 'date_publication_end_short']
    list_display_links = ['__str__']

    exclude = ('created_by', 'updated_by')
    readonly_fields = ('date_created', 'date_updated', 'date_approved')
    actions = ['make_draft', 'make_not_draft', 'make_approved', 'make_not_approved',
               'make_published', 'make_not_published']
    save_on_top = True
    actions_on_bottom = True
    mptt_indent_field = '__str__'

    page_fields = [
        ('name', ),
        ('ptype', ),
        ('parent', ),
        ('template', ),
    ]

    if hasattr(settings, 'SITE_ID'):
        if settings.PAGES_USE_SITE_ID:
            page_fields.append(('sites', ))

    fieldsets = [
        (None, {'fields': page_fields}),


        (_('State'), {'fields': [
            ('is_login_required', 'is_permission_required'),
            ('is_draft', 'is_approved', 'is_published'),
            ('date_created', 'date_updated', 'date_approved', ),
            ('date_publication', 'date_publication_end', ),
            ('comment',),
        ], 'classes': ['collapse']}),
    ]

    def get_default_ptype(self):
        return 'page'

    def get_form(self, request, obj=None, **kwargs):
        # just save obj reference for future processing in Inline
        request._obj_ = obj
        form = super(PageAdmin, self).get_form(request, obj, **kwargs)
        form.base_fields['ptype'].initial = self.get_default_ptype()
        return form

    def get_all_content_inlines(self, object_id):
        inlines = []
        result = PageContent.objects.filter(page=object_id)
        content_types = [o.type for o in result]
        all_content_types = PageContentType.objects.all()
        content_class = None
        inlines.append(PageContentInline)
        for content_type in all_content_types:
            if content_type in content_types:
                class_name = content_type.admin_class_name
                try:
                    content_class = getattr(pagecontenttypes, class_name)
                    if not (issubclass(content_class, admin.TabularInline) or
                            issubclass(content_class, admin.StackedInline)):
                        content_class = None
                except AttributeError:
                    try:
                        if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
                            for module in PAGE_EXT_CONTENT_INLINES:
                                if hasattr(module, class_name):
                                    content_class = getattr(module, class_name)
                                    if not (issubclass(content_class, admin.TabularInline) or
                                            issubclass(content_class, admin.StackedInline)):
                                        content_class = None
                                    else:
                                        break
                    except AttributeError:
                        pass
                if content_class is not None:
                    inlines.append(content_class)
            content_class = None
        return inlines

    def change_view(self, request, object_id, form_url='', extra_context=None):
        self.inlines = self.get_all_content_inlines(object_id)
        return super(PageAdmin, self).change_view(request, object_id)

    def add_view(self, request, form_url='', extra_context=None):
        self.inlines = [PageContentInline]
        return super(PageAdmin, self).add_view(request)

    @staticmethod
    def __get_message_bit(rows_updated):
        if rows_updated == 1:
            message_bit = _('one pages was')
        else:
            message_bit = _('{0} pages were').format(rows_updated)
        return message_bit

    def make_draft(self, request, queryset):
        rows_updated = queryset.update(is_draft=True, is_approved=False, date_approved=None, is_published=False)
        self.message_user(request, _('{0} successfully marked as draft.').format(self.__get_message_bit(rows_updated)))
    make_draft.short_description = _('Mark selected pages as draft')

    def make_not_draft(self, request, queryset):
        rows_updated = queryset.update(is_draft=False)
        self.message_user(request, _('{0} successfully marked as not draft.').format(self.__get_message_bit(rows_updated)))
    make_not_draft.short_description = _('Mark selected pages as not draft')

    def make_approved(self, request, queryset):
        rows_updated = queryset.update(is_approved=True, date_approved=timezone.now, is_draft=False)
        message = _('{0} successfully marked as approved.').format(self.__get_message_bit(rows_updated))
        self.message_user(request, message)
    make_approved.short_description = _('Mark selected pages as approved')

    def make_not_approved(self, request, queryset):
        rows_updated = queryset.update(is_approved=False, date_approved=None, is_published=False)
        message = _('{0} successfully marked as not approved.').format(self.__get_message_bit(rows_updated))
        self.message_user(request, message)
    make_not_approved.short_description = _('Mark selected pages as not approved')

    def make_published(self, request, queryset):
        rows_updated = queryset.update(is_published=True, is_draft=False)
        message = _('{0} successfully marked as piblished.').format(self.__get_message_bit(rows_updated))
        self.message_user(request, message)
    make_published.short_description = _('Mark selected pages as published')

    def make_not_published(self, request, queryset):
        rows_updated = queryset.update(is_published=False)
        message = _('{0} successfully marked as not published.').format(self.__get_message_bit(rows_updated))
        self.message_user(request, message)
    make_not_published.short_description = _('Mark selected pages as not published')

    def date_created_short(self, obj):
        return obj.date_created.strftime('%Y/%m/%d %H:%M:%S')

    date_created_short.short_description = _('Created')
    date_created_short.admin_order_field = 'date_created'

    def date_updated_short(self, obj):
        return obj.date_updated.strftime('%Y/%m/%d %H:%M:%S')

    date_updated_short.short_description = _('Updated')
    date_updated_short.admin_order_field = 'date_updated'

    def date_approved_short(self, obj):
        date = None
        if obj.date_approved is not None:
            date = obj.date_approved.strftime('%Y/%m/%d %H:%M:%S')
        return date

    date_approved_short.short_description = _('Approved')
    date_approved_short.admin_order_field = 'date_approved'

    def date_publication_short(self, obj):
        date = None
        if obj.date_publication is not None:
            date = obj.date_publication.strftime('%Y/%m/%d %H:%M:%S')
        return date

    date_publication_short.short_description = _('Publication')
    date_publication_short.admin_order_field = 'date_publication'

    def date_publication_end_short(self, obj):
        date = None
        if obj.date_publication_end is not None:
            date = obj.date_publication_end.strftime('%Y/%m/%d %H:%M:%S')
        return date

    date_publication_end_short.short_description = _('Publication end')
    date_publication_end_short.admin_order_field = 'date_publication_end'

    def save_model(self, request, obj, form, change):
        """

        """
        if not change:
            obj.created_by = request.user

        obj.updated_by = request.user
        obj.save()

admin.site.register(Page, PageAdmin)



"""
Django migrations for pages app

This package does not contain South migrations.
"""

SOUTH_ERROR_MESSAGE = """\n
South migrations not supported!
"""

# Ensure the user is not using Django 1.6 or below with South
try:
    # noinspection PyUnresolvedReferences
    from django.db import migrations
except ImportError:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured(SOUTH_ERROR_MESSAGE)



"""Implements page content models"""

from __future__ import unicode_literals
import importlib
import slugify
import itertools

from django.db import models
from django.utils.translation import ugettext_lazy as _

from markitup.fields import MarkupField

from pages.conf import settings
from pages.models.pagebasecontent import PageBaseContent

PAGE_EXT_CONTENT_TYPES = []
PAGE_MAX_SLUG_LENGTH = 245

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
    if settings.PAGES_PAGE_EXT_CONTENT_TYPES is not None:
        try:
            for content_type in settings.PAGES_PAGE_EXT_CONTENT_TYPES:
                PAGE_EXT_CONTENT_TYPES.append(importlib.import_module(content_type))
        except ImportError as e:
            raise Exception('Extended content type import error: {0}'.format(e))


class PageSlugContent(PageBaseContent):
    slug = models.CharField(max_length=PAGE_MAX_SLUG_LENGTH)
    objects = models.Manager()

    @property
    def full_slug(self):
        return self.language + '/' + self.slug

    def update_fields(self, change):
        if not change:
            self.type = 'slug'

    def save(self, *args, **kwargs):
        title = self.slug
        if settings.PAGES_PAGE_USE_META_TITLE_FOR_SLUG:
            try:
                meta = PageMetaContent.objects.get(page=self.page, language=self.language)
                title = meta.title
            except PageMetaContent.DoesNotExist:
                pass
        if not len(title):
            title = self.page.name
        self.slug = orig = slugify.slugify(
                title, only_ascii=settings.PAGES_PAGE_ONLY_ASCII_SLUGS
        )[:PAGE_MAX_SLUG_LENGTH]
        for x in itertools.count(1):
            if not PageSlugContent.objects.exclude(
                    page=self.page
            ).filter(slug=self.slug, language=self.language).exists():
                break
            self.slug = '{0:s}-{1:d}'.format(orig[:PAGE_MAX_SLUG_LENGTH - len(str(x)) - 1], x)
        super(PageSlugContent, self).save(*args, **kwargs)

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Slug')
        verbose_name_plural = _('Slugs')
        unique_together = ('language', 'page',)

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'slug'
        multiple_per_page = True
        multiple_per_locale = False


class PageRedirectContent(PageBaseContent):
    redirect_to_page = models.CharField(max_length=254, null=True, blank=True)
    redirect_to_url = models.URLField(max_length=254, null=True, blank=True)
    is_permanent = models.BooleanField(_('Permanent'), default=False)

    def update_fields(self, change):
        if not change:
            self.type = 'redirect'

    def get_redirect_url(self, request):
        url = None
        if self.redirect_to_page:
            url = request.build_absolute_uri(self.redirect_to_page)
        else:
            if self.redirect_to_url:
                url = self.redirect_to_url
        return url

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Redirect')
        verbose_name_plural = _('Redirect')

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'redirect'
        multiple_per_page = True
        multiple_per_locale = False


class PageMetaContent(PageBaseContent):
    title = models.CharField(max_length=160, blank=True)
    description = models.TextField(max_length=160, blank=True)
    keywords = models.CharField(max_length=200, blank=True)
    is_noindex = models.BooleanField(_('NoIndex'), default=False)
    is_nofollow = models.BooleanField(_('NoFollow'), default=False)
    objects = models.Manager()

    def update_fields(self, change):
        if not change:
            self.type = 'meta'

    @property
    def robots(self):
        """Return values for robots html meta key"""
        r = 'noindex' if self.is_noindex else 'index'
        r += ','
        r += 'nofollow' if self.is_nofollow else 'follow'
        return r

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Meta')
        verbose_name_plural = _('Meta')
        unique_together = ('language', 'page',)

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'meta'
        multiple_per_page = True
        multiple_per_locale = False


class PageTextContent(PageBaseContent):
    text = models.TextField(blank=True)
    is_template = models.BooleanField(_('Template?'), default=False)
    objects = models.Manager()

    def update_fields(self, change):
        if not change:
            self.type = 'text'

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Text')
        verbose_name_plural = _('Text')

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'text'
        multiple_per_page = True
        multiple_per_locale = True


class PageMarkdownContent(PageBaseContent):
    text = MarkupField(blank=True)
    is_template = models.BooleanField(_('Template?'), default=False)
    objects = models.Manager()

    def update_fields(self, change):
        if not change:
            self.type = 'markdown'

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Markdown')
        verbose_name_plural = _('Markdown')

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'markdown'
        multiple_per_page = True
        multiple_per_locale = True



import django

from mptt.managers import TreeManager, TreeQuerySet

from pages.conf import settings


class PageManager(TreeManager):

    def get_queryset(self, *args, **kwargs):
        """
        Ensures that this manager always returns nodes in tree order.
        """
        if django.VERSION < (1, 7):
            qs = TreeQuerySet(self.model, using=self._db)
        else:
            qs = super(TreeManager, self).get_queryset(*args, **kwargs)

        # Restrict operations to pages on the current site if needed
        if settings.PAGES_HIDE_SITES and settings.PAGES_USE_SITE_ID:
            return qs.order_by(self.tree_id_attr, self.left_attr).filter(sites=settings.SITE_ID)
        else:
            return qs.order_by(self.tree_id_attr, self.left_attr)

    def on_site(self, site_id=None):
        """Return a :class:`QuerySet` of pages that are published on the site
        defined by the ``SITE_ID`` setting.

        :param site_id: specify the id of the site object to filter with.
        """
        if settings.PAGES_USE_SITE_ID:
            if not site_id:
                site_id = settings.SITE_ID
            return self.get_queryset().filter(sites=site_id)
        return self.all()

    def published(self):
        """Return queryset with only published pages"""
        return self.get_queryset().filter(is_published=True, is_hidden=False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import embed_video.fields
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PageVideoContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=100, verbose_name='Type', db_index=True)),
                ('language', models.CharField(default=b'en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(default=True, verbose_name='Extended?')),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created')),
                ('date_updated', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Updated')),
                ('video', embed_video.fields.EmbedVideoField(blank=True)),
                ('title', models.CharField(max_length=160, blank=True)),
                ('description', models.TextField(max_length=160, blank=True)),
                ('created_by', models.ForeignKey(related_name='extpages_pagevideocontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('updated_by', models.ForeignKey(related_name='extpages_pagevideocontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Video',
                'verbose_name_plural': 'Video',
            },
            bases=(models.Model,),
        ),
    ]



"""Implements PageContent model"""

from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


from pages.models import Page
from pages.models.pagecontenttype import PageContentType


@python_2_unicode_compatible
class PageContent(models.Model):
    page = models.ForeignKey(Page, verbose_name=_('Page'))
    type = models.ForeignKey(PageContentType)
    objects = models.Manager()

    def __str__(self):
        """id string of instance"""
        return '{0}:{1}:{2}'.format(self.page, self.type, self.pk)

    def save(self, *args, **kwargs):
        """Override the default ``save`` method."""

        # Call parent's ``save`` method
        super(PageContent, self).save(*args, **kwargs)

    class Meta:
        app_label = 'pages'
        verbose_name = _('Content')
        verbose_name_plural = _('Content')
        unique_together = ('page', 'type',)



import django
from django.db import models

from pages.conf import settings


class ContentTypeManager(models.Manager):

    def get_queryset(self):
        super_ = super(ContentTypeManager, self)
        # fix get_query_set for old Django releases
        if django.VERSION < (1, 6):
            qs = super_.get_query_set()
        else:
            qs = super_.get_queryset()
        if not settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
            return qs.filter(is_extended=False)
        else:
            return qs

    get_query_set = get_queryset

    def get_queryset_compat(self):
        get_queryset = (self.get_query_set if hasattr(self, 'get_query_set') else self.get_queryset)
        return get_queryset()

    def extended(self):
        """Return queryset with only extended content types"""
        return self.get_queryset_compat().filter(is_extended=True)



"""Django urls for content management system."""

from django.conf.urls import url

from pages.views import PageDetailsView

app_name = 'pages'
urlpatterns = [url(r'^(?P<slug>[-\w]+)/$', PageDetailsView.as_view(), name='show')]



"""Implements admin interface for Pages"""

from django.contrib import admin

from pages.models import PageContentType


class PageContentTypeAdmin(admin.ModelAdmin):
    list_display = ['__str__']
    list_display_links = ['__str__']
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageContentType, PageContentTypeAdmin)



"""Implements page details view for content management system"""

from datetime import datetime
from hashlib import md5

from django.http import HttpResponseForbidden, Http404
from django.http import HttpResponseRedirect, HttpResponsePermanentRedirect
from django.views.generic import TemplateView
from django.views.decorators.cache import cache_control
from django.views.decorators.http import condition
from django.shortcuts import render
from django.utils import translation
from django.utils.translation import get_language
from django.utils import timezone
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse

from guardian.shortcuts import get_perms
from appcore.views.mixins import DecoratorChainingMixin

from pages.conf import settings
from pages.cache import cache
from pages.models import Page
from pages.models import PageContentType
from pages.models import PageSlugContent
from pages.models import PageRedirectContent


def get_timestamp(slug, language):
    # try get page modification timestamp for http caching
    page_cache_version_key = settings.PAGES_PAGE_VERSION_KEY + language + ':' + slug
    timestamp_cache_key = settings.PAGES_PAGE_CACHE_KEY + language + ':' + slug + '_timestamp'
    cache_version = cache.get(page_cache_version_key)
    cache_version = 1 if cache_version is None else cache_version
    timestamp = cache.get(timestamp_cache_key, version=cache_version)
    if timestamp is None:
        timestamp = timezone.now().strftime('%Y-%m-%d %H:%M:%S')
        cache.set(timestamp_cache_key, timestamp, settings.PAGES_PAGE_CACHE_TIMEOUT, version=cache_version)
    return timestamp


def get_etag(request, **kwargs):
    is_authenticated = str(request.user.is_authenticated())
    slug = kwargs.get('slug', None)
    language = get_language()
    modified = get_timestamp(slug, language)
    etag_bytes = (modified + language + is_authenticated).encode('utf-8')
    return md5(etag_bytes).hexdigest()


def get_last_modified(request, **kwargs):
    slug = kwargs.get('slug', None)
    language = get_language()
    modified = get_timestamp(slug, language)
    return datetime.strptime(modified, '%Y-%m-%d %H:%M:%S')


class PageDetailsView(DecoratorChainingMixin, TemplateView):
    decorators = [cache_control(must_revalidate=True, proxy_revalidate=True, max_age=settings.PAGES_PAGE_HTTP_MAX_AGE),
                  condition(etag_func=get_etag, last_modified_func=get_last_modified)]

    def get(self, request, **kwargs):
        is_authenticated = str(request.user.is_authenticated())
        slug = kwargs.get(settings.PAGES_PAGE_SLUG_NAME, None)
        if slug is None:
            raise Http404

        language = get_language()

        if settings.PAGES_USE_FALLBACK_LANGUAGE:
            use_language = request.COOKIES.get(settings.LANGUAGE_COOKIE_NAME, None)
            use_fallback_language = request.COOKIES.get(settings.PAGES_FALLBACK_LANGUAGE_COOKIE_NAME, None)
            if use_fallback_language is not None and use_language != language:
                translation.activate(settings.PAGES_FALLBACK_LANGUAGE)
                if slug == settings.PAGES_HOME_PAGE_SLUG:
                    response = HttpResponseRedirect('/')
                else:
                    response = HttpResponseRedirect(reverse('pages:show', args=(slug,)))
                response.set_cookie(settings.LANGUAGE_COOKIE_NAME, settings.PAGES_FALLBACK_LANGUAGE)
                return response

        page_cache_key = settings.PAGES_PAGE_CACHE_KEY + language + ':' + slug + ':' + is_authenticated
        page_cache_version_key = settings.PAGES_PAGE_VERSION_KEY + language + ':' + slug

        # try get cached page
        cache_version = cache.get(page_cache_version_key)
        cache_version = 1 if cache_version is None else cache_version
        page = cache.get(page_cache_key, version=cache_version)

        if page:
            # check if login required
            if page.is_login_required and not request.user.is_authenticated():
                return HttpResponseRedirect(settings.LOGIN_URL)

            # check user view permission
            if page.is_permission_required:
                if 'view_page' not in get_perms(request.user, page):
                    if settings.PAGES_RAISE_403:
                        raise PermissionDenied
                    if settings.PAGES_RENDER_403:
                        return render(request, settings.PAGES_TEMPLATE_403, {})
                    else:
                        return HttpResponseForbidden()

            # try get pages content from cache
            page_redirect = cache.get(page_cache_key + 'redirect', version=cache_version)
            if page_redirect:
                if page_redirect['permanent']:
                    return HttpResponsePermanentRedirect(page_redirect['url'])
                else:
                    return HttpResponseRedirect(page_redirect['url'])

            page_content = cache.get(page_cache_key + 'content', version=cache_version)
            page_ext_content = cache.get(page_cache_key + 'ext_content', version=cache_version)

        else:
            # try request pages from db and store pages content in the cache
            slugs = PageSlugContent.objects.filter(slug=slug, language=language)
            if not slugs:
                if settings.PAGES_USE_FALLBACK_LANGUAGE:
                    # try use fallback language for requested page
                    if settings.PAGES_FALLBACK_LANGUAGE != language:
                        slugs = PageSlugContent.objects.filter(slug=slug, language=settings.PAGES_FALLBACK_LANGUAGE)
                    if slugs:
                        translation.activate(settings.PAGES_FALLBACK_LANGUAGE)
                        if slugs[0].slug == settings.PAGES_HOME_PAGE_SLUG:
                            response = HttpResponseRedirect('/')
                        else:
                            response = HttpResponseRedirect(reverse('pages:show', args=(slugs[0].slug,)))
                        response.set_cookie(settings.LANGUAGE_COOKIE_NAME, settings.PAGES_FALLBACK_LANGUAGE)
                        response.set_cookie(settings.PAGES_FALLBACK_LANGUAGE_COOKIE_NAME, settings.PAGES_FALLBACK_LANGUAGE)
                        return response
                raise Http404

            slug = slugs[0]
            try:
                page = Page.objects.published().filter(pk=slug.page_id)[0]

                # cache pages content
                cache.set(page_cache_version_key, cache_version, settings.PAGES_PAGE_CACHE_TIMEOUT)
                cache.set(page_cache_key, page, settings.PAGES_PAGE_CACHE_TIMEOUT, version=cache_version)

                # check if login required
                if page.is_login_required and not request.user.is_authenticated():
                    return HttpResponseRedirect(settings.LOGIN_URL)

                # check user view permission
                if page.is_permission_required:
                    if 'view_page' not in get_perms(request.user, page):
                        if settings.PAGES_RAISE_403:
                            raise PermissionDenied
                        if settings.PAGES_RENDER_403:
                            return render(request, settings.PAGES_TEMPLATE_403, {})
                        else:
                            return HttpResponseForbidden()

                page_content = {}
                page_ext_content = {}
                page_redirect = {}

                # check if this pages has redirect
                try:
                    redirect = PageRedirectContent.objects.get(page=page, language=language)
                    page_redirect.update({
                        'url': redirect.get_redirect_url(request),
                        'permanent': redirect.is_permanent
                    })
                    cache.set(page_cache_key + 'redirect', page_redirect,
                              settings.PAGES_PAGE_CACHE_TIMEOUT, version=cache_version)
                    if page_redirect['permanent']:
                        return HttpResponsePermanentRedirect(page_redirect['url'])
                    else:
                        return HttpResponseRedirect(page_redirect['url'])

                except PageRedirectContent.DoesNotExist:
                    content_types = PageContentType.objects.filter(is_extended=False)
                    for content_type in content_types:
                        content_class = Page.get_content_class(content_type.type)
                        content = content_class.objects.filter(page=page, language=language)
                        if content:
                            page_content.update({content_type.type: content})

                    if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
                        ext_content_types = PageContentType.objects.extended()
                        for content_type in ext_content_types:
                            content_class = Page.get_content_class(content_type.type)
                            if content_class:
                                ext_content = content_class.objects.filter(page=page, language=language)
                                if ext_content:
                                    page_ext_content.update({content_type.type: ext_content})
                if page_content:
                    cache.set(page_cache_key + 'content', page_content,
                              settings.PAGES_PAGE_CACHE_TIMEOUT, version=cache_version)
                if page_ext_content:
                    cache.set(page_cache_key + 'ext_content', page_ext_content,
                              settings.PAGES_PAGE_CACHE_TIMEOUT, version=cache_version)

            except IndexError:
                raise Http404

        context = self.get_context_data(**kwargs)
        context.update({
            'page': {
                'page': page,
                'slug': slug,
                'content': page_content,
                'ext_content': page_ext_content,
                'cache_key': page_cache_key,
                'cache_version': cache_version,
                'timeout': settings.PAGES_PAGE_CACHE_TIMEOUT,
            }
        })

        return render(request, page.get_template(), context)



"""Implements admin interface for page content"""

from django.contrib import admin

from pages.models import PageSlugContent
from pages.models import PageRedirectContent
from pages.models import PageMetaContent
from pages.models import PageTextContent
from pages.models import PageMarkdownContent


class PageSlugContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True


admin.site.register(PageSlugContent, PageSlugContentAdmin)


class PageSlugContentInline(admin.StackedInline):
    model = PageSlugContent
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated', )
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('slug', ),
            ('comment', ),
        ]}),
    ]


class PageRedirectContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by')
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageRedirectContent, PageRedirectContentAdmin)


class PageRedirectContentInline(admin.StackedInline):
    model = PageRedirectContent
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('redirect_to_page', ),
            ('redirect_to_url', ),
            ('is_permanent', ),
            ('comment', ),
        ]}),
    ]


class PageMetaContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ['type', 'sid', 'is_extended', 'created_by', 'updated_by']
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageMetaContent, PageMetaContentAdmin)


class PageMetaContentInline(admin.StackedInline):
    model = PageMetaContent
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('title', ),
            ('keywords', ),
            ('description', ),
            ('is_noindex', 'is_nofollow', ),
            ('comment', ),
        ]}),
    ]


class PageTextContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageTextContent, PageTextContentAdmin)


class PageTextContentInline(admin.StackedInline):
    model = PageTextContent
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated', )
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('text', ),
            ('is_template', ),
            ('comment', ),
        ]}),
    ]


class PageMarkdownContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageMarkdownContent, PageMarkdownContentAdmin)


class PageMarkdownContentInline(admin.StackedInline):
    model = PageMarkdownContent
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated', )
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('text', ),
            ('is_template', ),
            ('comment', ),
        ]}),
    ]



from pages.models.page import Page
from pages.models.page import PageUserObjectPermission, PageGroupObjectPermission
from pages.models.pagecontent import PageContent
from pages.models.pagecontenttype import PageContentType
from pages.models.pagecontenttypes import PageSlugContent
from pages.models.pagecontenttypes import PageRedirectContent
from pages.models.pagecontenttypes import PageMetaContent
from pages.models.pagecontenttypes import PageTextContent
from pages.models.pagecontenttypes import PageMarkdownContent



VERSION = (0, 2, 0)


def get_version():
    """Returns the version as a human-format string."""
    return '.'.join([str(i) for i in VERSION])

__author__ = 'dlancer'
__docformat__ = 'restructuredtext en'
__copyright__ = 'Copyright 2014-2016, dlancer'
__license__ = 'BSD'
__version__ = get_version()
__maintainer__ = 'dlancer'
__email__ = 'dmdpost@gmail.com'
__status__ = 'Development'
